###############################################################################################################
## This code reproduces the result of Section 3.2. of                                                        ##
## "Statistical methods for optimizing treatment decisions in psychiatry: A tutorial" (Petkova et al, 2019). ##
###############################################################################################################

## the depression dataset
dat_depression <- read.csv("depressionData.csv")
Tr <- dat_depression[,1]  # n-by-1 vector of treatment indicators; 1 for placebo, 2 for active drug. 
y <-  dat_depression[,2]  # n-by-1 vector of outcomes
X.tmp <- as.matrix(dat_depression[,-c(1,2)])   # X.temp is a n-by-p matrix of pretreatment covariates
X <- X.tmp[,-c(2, 6, 7, 8, 9, 10, 11)]  # exclude discrete-valued covariates (but we don't have to exclude these- SIMML can be applied to discrete covariates, as long as there is at least one continous covariate.)
X[,3] <- log(X[,3]+1)           # log transformation for this highly skewed covariate.


#install.packages("simml")
library(simml)

set.seed(1234)
## we can improve the efficiency of the SIMML estimator by using the efficiency augmentation term, mu.hat; the vector mu.hat can be estimated by a main effect only model (y~ X).)
# asume a linear model for the main effect of X. 
glm.fit <- glm(y ~ X)  # X needs to a matrix (not a data.frame)
mu.hat <- predict(glm.fit)

# estimate SIMML (the nonlinear GEM for estimating interactions)
simml.obj <- simml(y, Tr, X, mu.hat=mu.hat)
simml.obj$alpha.coef
## we can also obtain bootstrap CIs 
#simml.obj.boot <- simml(y, Tr, X, mu.hat=mu.hat, bootstrap =TRUE)  # 200 bootstrap replications (takes about 30 secs to run)
#simml.obj.boot$boot.ci


## visualization
simml.obj$alpha.coef      # the estimated single-index coefficients
g.fit <- simml.obj$g.fit  # the estimated treatment-specific link functions. note that "g.fit" is a mgcv::gam object.
plot(g.fit)


## we can improve visualization by using the package "mgcViz".
#install.packages("mgcViz")   # mgcViz depends on the package "rgl". On MacOS, "rgl" depends on XQuartz, which you can download from xquartz.org
library(mgcViz)
# transform the "mgcv::gam" object to a "mgcViz" object (to obtain an improved visualization)
g.fit <- getViz(simml.obj$g.fit)
yc.range <- range(y) - mean(y)

plot1  <- plot( sm(g.fit,1) )
plot1 + l_fitLine(colour = "red") + l_rug(mapping = aes(x=x, y=y), alpha = 0.8) +
  l_ciLine(mul = 5, colour = "blue", linetype = 2) +
  l_points(shape = 19, size = 1, alpha = 0.1) +ylim(yc.range[1], yc.range[2]) +
  xlab(expression(paste("z = ", alpha*minute, "x")))  +  ylab("Change in HRSD") + ggtitle("Placebo group (A=0)") #+  theme_classic()

plot2 <- plot( sm(g.fit,2) )
plot2 + l_fitLine(colour = "red") + l_rug(mapping = aes(x=x, y=y), alpha = 0.8) +
  l_ciLine(mul = 5, colour = "blue", linetype = 2) +
  l_points(shape = 19, size = 1, alpha = 0.1) +ylim(yc.range[1],yc.range[2]) +
  xlab(expression(paste("z = ", alpha*minute, "x"))) +ylab(" ") + ggtitle("Drug group (A=1)") #+ theme_classic()

trans = function(x) x + g.fit$coefficients[2]

plotDiff(s1 = sm(g.fit, 2), s2 = sm(g.fit, 1), trans=trans) +  l_ciPoly() +
  l_fitLine() + geom_hline(yintercept = 0, linetype = 2) +
  xlab(expression(paste("z = ", alpha*minute, "x")))  +
  ylab("(Drug effect) - (Placebo effect) ") +
  ggtitle("Contrast between two treatment effects")+
  geom_vline(xintercept=-0.45, linetype="dotted", color = "red", size=0.8) + theme_classic()


# another way of visualization, using ggplot2
library(ggplot2)
dat  <- data.frame(y= simml.obj$g.fit$model$y,
                   x= simml.obj$g.fit$model$single.index,
                   Treatment= factor(simml.obj$g.fit$model$Tr, levels=c("1","2"), labels=c("Placebo", "Active drug"))) 
g.plot <- ggplot(dat, aes(x=x, y=y, color=Treatment, shape=Treatment, linetype=Treatment)) +
   geom_point(aes(color=Treatment, shape=Treatment), size=1, fill="white") +
   scale_colour_brewer(palette="Set1", direction=-1) + theme_classic() +
   xlab(expression(paste("z = ", alpha*minute,"x"))) + ylab("Change in HRSD")
g.plot + geom_smooth(method=gam, formula= y~ s(x, bs=simml.obj$bs, k=simml.obj$k),
                     se=TRUE, fullrange=TRUE, alpha = 0.35)




## estimate SIMML based only on the 4 significant covariates (x1, x2, x9, x11) identified by "simml.obj.boot$boot.ci" (bootstrap CIs).
simml.obj.subset <- simml(y, Tr, X[,c(1,2,9,11)], mu.hat=mu.hat)
simml.obj.subset$alpha.coef
plot(simml.obj.subset$g.fit)


## Treatment decision rule (TDR) construction
simml.obj <- simml(y, Tr, X, mu.hat=mu.hat)
simml.obj$alpha.coef
# estimated treatment decision rule (for the training set)
pred.simml(simml.obj, newx= X)$trt.rule




## Value comparison
#install.packages("pirate")
library(pirate)  # for linear-GEM of Petkova et al (2017)

# we will estimate "Value" of TDRs using IPWE (calculated based on 500 randomly split independent testing data, whereas the TDRs are estimated based on training data, with a 5:1 training/testing split)
results.aggregated  <- NULL   # will store results here
n.rep <- 50 #500  # the number of random splits (will split the dataset to training/testing sets at 5:1 ratio)
n <- length(y)
for(rep.number in 1:n.rep)
{
  print(rep.number)

  set.seed(1234 + rep.number)
  folds <- sample(1:2, n, replace = TRUE, prob = c(0.83, 0.17))  # 5:1 training/testing set splitting
  y.train <- y[folds==1]
  y.test <- y[folds==2]
  X.train <- X[folds==1,]
  X.test <- X[folds==2,]
  Tr.train <- Tr[folds==1]
  Tr.test <- Tr[folds==2]


  # Estimate an efficiency augmentation vector, mu.hat, using a main effect only model (y~ X).
  glm.fit <- glm(y.train ~ X.train)
  mu.hat <- predict(glm.fit)


  # 1. simml with efficiency augmentation
  simml.fit <- simml(y.train, Tr.train, X.train, mu.hat=mu.hat)
  trt.rule <- pred.simml(simml.fit, newx=X.test)$trt.rule
  # Value estimation
  simml.value1 <-  mean(y.test[trt.rule == Tr.test])


  # 2. simml without efficiency augmentation
  simml.fit <- simml(y.train, Tr.train, X.train, mu.hat=NULL)
  trt.rule <- pred.simml(simml.fit, newx=X.test)$trt.rule
  # Value estimation
  simml.value2 <-  mean(y.test[trt.rule == Tr.test])


  # 3. simml with efficiency augmentation (with the selected covariates only)
  simml.fit <- simml(y.train, Tr.train, X.train[,c(1,2,9,11)], mu.hat=mu.hat)
  trt.rule <- pred.simml(simml.fit, newx=X.test[,c(1,2,9,11)])$trt.rule
  # Value estimation
  simml.value3 <-  mean(y.test[trt.rule == Tr.test])


  # 4. modified covariates method of Tien et al 2014
  mc <- 0.5*(-1)^as.numeric(Tr.train) *cbind(1, X.train)
  mc.coef  <-  coef(glm(y.train ~ mc))
  trt.rule <- (cbind(1, X.test) %*% mc.coef[-1] > 0) +1
  # Value estimation
  mc.value  <-  mean(y.test[trt.rule == Tr.test])


  # For the linear GEM methods of Petkova et al (2017).
  dat.train <- data.frame(Tr.train, y.train, X.train)
  dat.test <-  data.frame(Tr.test, y.test, X.test)

  model_F <- gem_fit(dat = dat.train, method = "F")     # F-criterion
  model_nu <- gem_fit(dat = dat.train, method = "nu")   # Numerator-criterion
  model_de <- gem_fit(dat = dat.train, method = "de")   # Denominator-criterion

  lgem.F.value <- gem_test_sample(dat.test, model_F[[2]])$PAB_gem
  lgem.nu.value <- gem_test_sample(dat.test, model_nu[[2]])$PAB_gem
  lgem.de.value <- gem_test_sample(dat.test, model_de[[2]])$PAB_gem
  unres.value <- gem_test_sample(dat.test, model_de[[2]])$PAB_unres     # unrestricted linear models

  all.placebo <- mean(y.test[Tr.test == 1])       # treating everyone with placebo
  all.placebo
  all.drug <- mean(y.test[Tr.test == 2])          # treating everyone with the drug
  all.drug


  results.ii <- c(simml.value1, simml.value2, simml.value3, mc.value,
                  lgem.F.value, lgem.nu.value, lgem.de.value, unres.value,
                  all.placebo, all.drug)

  print(results.ii)
  results.aggregated <- rbind(results.aggregated, results.ii)
}


colnames(results.aggregated) <-  c("SIMML(eff)", "SIMML(no eff)", "SIMML(subset)", "MC", "linGEM.F", "linGEM.nu",
                                   "linGEM.de", "unres", "all PBO", "all DRUG")

apply(results.aggregated, 2, mean)
apply(results.aggregated, 2, sd)
apply(results.aggregated, 2, function(x) quantile(x, probs= c(0.05, 0.95)))



######################################################################
## END OF THE FILE
######################################################################
