######################################################################################################################################################
## Example 1 (the ParentCorps Study) from "Statistical methods for optimizing treatment decisions in psychiatry: A tutorial" (Petkova et al, 2019)  ##
######################################################################################################################################################

dat_ache <- read.csv("achevementData.csv")
A <- dat_ache[,1]  # treatment indicator;  ParentCorps (A = 1) vs. control (A = 0) 
y <- dat_ache[,2]  # outcome variable 
X <- dat_ache[,-c(1,2)]  # covariates 
# standardize X 
Xc <- as.matrix(scale(X))
# dat is a data frame with first column as the treatment index, second column as the outcome, and the remaining columns as the covariates design matrix.
dat <- data.frame(A=A, y=y, Xc=Xc)


# fit the linear GEM model 
library(pirate)
model_ache <- gem_fit(dat = dat, method = "F")
# alpha (GEM) coefficient (the calculated weight α for combining the predictors X)
model_ache$gem$alpha
rownames(model_ache$gem$alpha) <- colnames(X)
model_ache$gem$alpha

# the intercepts and slopes associaetd with the GEM variable z=α'X for the two treatment groups
model_ache$gem$GammaByTrt
rownames(model_ache$gem$GammaByTrt) <- c("Control", "ParentCorps")
colnames(model_ache$gem$GammaByTrt) <- c("int.", "slope")
model_ache$gem$GammaByTrt


# fit individual models separately for each of the p = 6 risk factors including treatment assignment and interaction
# compute p-values of the F-test for significance of the interaction terms for these potential modifiers of ParentCorps treatment effect
summary(lm(y~Xc[,1]*A))
summary(lm(y~Xc[,2]*A))
summary(lm(y~Xc[,3]*A))
summary(lm(y~Xc[,4]*A))
summary(lm(y~Xc[,5]*A))
summary(lm(y~Xc[,6]*A))
# none of the interactions terms were statistically significant 


# fit a full unrestricted regression model with all predictors and their interactions with treatment 
# and a reduced model without the interactions, 
# and performed an F -test for significance of the full set of interactions
full.model  <- lm(y~Xc*A)
reduced.model <- lm(y~Xc + A)
anova(reduced.model, full.model)
# F(6,739) = 1.26, p-value=0.274 

summary(full.model)


# Based on the permutation testing approach to control type I error rates, described in Section (2.4), 
# the resulting GEM has a statistically significant interaction with the treatment indicator (p = 0.002) 
## library(plyr)
permuteN <- 500
colnames(dat)[1] <- "trt"
K <- length(unique(dat$trt))
dat[, 1] <- factor(dat[, 1], labels = c(1:K))
N <- ddply(dat, .(dat$trt), nrow)[, 2]
mm <- gem_fit(dat, method="F")
p <- mm[[3]]
set.seed(1)
for (i in 1:permuteN) {
  datt <- dat
  datt[, 1] <- sample(dat$trt)
  mmm <- gem_fit(datt, method="F")
  p <- c(p, mmm[[3]])
}
p_value <- sum(p[2:(1 + permuteN)] <= p[1])/permuteN
p_value

# alternatively, run: 
set.seed(1);  permute_pvalue(dat, permuteN=500, method = "F")$perm_p


# plot (Figure 1)
#model_ache$plot
z <- Xc %*% model_ache$gem$alpha
dat$z <- z
dat$A <- factor(A, levels=c("0", "1"), labels =c("Control","ParentCorps"))
p1 <- ggplot(dat, aes_string(x = "z", y = "y", group = "A", color = "A", shape= "A")) + geom_point(size=0.95) + geom_smooth(method = "lm",fullrange = T)
p1 +  theme(legend.title=element_blank())
z1 <- (model_ache$gem$GammaByTrt[2]-model_ache$gem$GammaByTrt[1])/(model_ache$gem$GammaByTrt[3]-model_ache$gem$GammaByTrt[4])
z1
# -1.26
p1 + theme(legend.title=element_blank()) + xlab('GEM') + ylab("Academic achievement at post") + geom_vline(xintercept = z1, linetype="dashed", color = "red", size=0.5)
#xlab(expression(paste("z = ", alpha*minute,"x"))) 





 # estimate "Value" of TDRs using IPWE (calculated based on 500 randomly split independent testing data, whereas the TDRs are estimated based on training data, with a 5:1 training/testing split)
 results.aggregated  <- NULL   # will store results here
 n.rep <- 500   # the number of random splits (will split the dataset to training/testing sets)
 n <- length(y)
 
 for(rep.number in 1:n.rep)
 {
   print(rep.number)
   
   set.seed(12 + rep.number)
   folds <- sample(1:2, n, replace = TRUE, prob = c(0.95, 0.05))  # training/testing set splitting
   y.train <- y[folds==1]
   y.test <- y[folds==2]
   X.train <- Xc[folds==1,]
   X.test <- Xc[folds==2,]
   A.train <- A[folds==1]
   A.test <- A[folds==2]
   
   # For the linear GEM methods of Petkova et al (2017).
   dat.train <- data.frame(A.train, y.train, X.train)
   dat.test <-  data.frame(A.test, y.test, X.test)
   
   model_F <- gem_fit(dat = dat.train, method = "F")     # F-criterion (can also do "nu" or "de")
   lgem.F.value <- gem_test_sample(dat.test, model_F[[2]])$PAB_gem
   #unres.value <- gem_test_sample(dat.test, model_F[[2]])$PAB_unres     # unrestricted linear models
   
   all.control <- mean(y.test[A.test == 0])       # treating everyone with control
   all.control
   all.ParentCorps <- mean(y.test[A.test == 1])   # treating everyone with ParentCorps
   all.ParentCorps
   
   results.ii <- c(lgem.F.value, #unres.value, 
                   all.control, all.ParentCorps)
   
   print(results.ii)
   results.aggregated <- rbind(results.aggregated, results.ii)
 }
 
 colnames(results.aggregated) <-  c("linGEM.F", #"unres", 
                                    "all.control", "all.ParentCorps")
 apply(results.aggregated, 2, mean)
 apply(results.aggregated, 2, sd)
 apply(results.aggregated, 2, function(x) quantile(x, probs= c(0.05, 0.95)))
 
 
 ######################################################################
 ## END OF THE FILE
 ######################################################################
 